package ru.ifmo.tracklayer.co;

import com.evelopers.unimod.runtime.ControlledObject;
import com.evelopers.unimod.runtime.context.StateMachineContext;

import ru.ifmo.tracklayer.ep.TracklayerEventProvider;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;

public class Tracklayer implements ControlledObject {
	private final static int NO_TRACK     = 0;
	private final static int GOOD_TRACK   = 1;
	private final static int BAD_TRACK    = 2;
	private final static int END_OF_TRACK = 3;
	private final static int DEPOT        = 4;
	private static int[] railroad;
	private static int position;
	private int storeCapacity;
	private static int tracksInStore;
	private static boolean inited = false;
	private static final int trackSize = 50;
	
	public Tracklayer() {
		Math.rint(System.currentTimeMillis());
		railroad = new int[30];
		Init();
		inited = true;
	}
	
	public void Init(){
		for(int i = 0; i < railroad.length; ++i) {
			railroad[i] = (int) (Math.random() * 3); 
		}
		railroad[0] = DEPOT;
		railroad[1] = DEPOT;
		railroad[railroad.length - 1] = END_OF_TRACK;
		position = 1;
		tracksInStore = 5;
		storeCapacity = 5;
	}
	
	/**
	*@unimod.action.descr Number of tracks in store
	*/
	public int x1 (StateMachineContext context)
	{
		return tracksInStore;
	}
	
	/**
	*@unimod.action.descr Move forward
	*/
	public void z1 (StateMachineContext context) {
		if (railroad[position + 1] == GOOD_TRACK) {	
			++position;
			TracklayerEventProvider.operationComplete();
		} else { 
			switch(railroad[position + 1]) {
			case GOOD_TRACK:
				break;
			case BAD_TRACK:
				TracklayerEventProvider.badFound();
				break;
			case NO_TRACK:
				TracklayerEventProvider.noneFound();
				break;
			case END_OF_TRACK:
				TracklayerEventProvider.endFound();
				break;
			}
		}
	}
	
	/**
	*@unimod.action.descr Move backward
	*/
	public void z2 (StateMachineContext context) {
		if (position > 1) {	
			--position;
			TracklayerEventProvider.operationComplete();
		} else {
			TracklayerEventProvider.endFound();
		}
	}
	
	/**
	*@unimod.action.descr Put track on 
	*/
	public void z3 (StateMachineContext context)
	{	
		if (railroad[position + 1] != GOOD_TRACK) {;
			--tracksInStore;
			railroad[position + 1] = GOOD_TRACK;
		}
		TracklayerEventProvider.operationComplete();
	}
	
	/**
	*@unimod.action.descr Take track off 
	*/
	public void z4 (StateMachineContext context)
	{	
		railroad[position + 1] = NO_TRACK;	
		TracklayerEventProvider.operationComplete();
	}
	
	/**
	*@unimod.action.descr Load matherials
	*/
	public void z5 (StateMachineContext context)
	{	
		if (tracksInStore >= storeCapacity) {
			TracklayerEventProvider.storeIsFull();
		} else {
			++tracksInStore;
			TracklayerEventProvider.operationComplete();
		};
	}
	
	/**
	*@unimod.action.descr Init tracklayer
	*/
	public void z6 (StateMachineContext context)
	{	
		Init();
		TracklayerEventProvider.operationComplete();
	}
	
	private static void drawTrack(int trackType, int x, int y, Graphics g) {
		switch(trackType) {
		case GOOD_TRACK:
			g.setColor(Color.GREEN);
			break;
		case BAD_TRACK:
			g.setColor(Color.RED);
			break;
		case NO_TRACK:
			g.setColor(Color.WHITE);
			break;
		case END_OF_TRACK:
		case DEPOT:
			g.setColor(Color.BLACK);
			break;
		}
		g.fillRect(x,y,trackSize,trackSize);
		if (trackType == END_OF_TRACK) {
			g.fillRect(x,y - trackSize,trackSize / 3,trackSize);
		}
		
		if (trackType == DEPOT) {
			g.fillRect(x + trackSize / 3,y - trackSize * 2,trackSize / 3,trackSize * 2);
			g.fillRect(x, y - trackSize * 2 - trackSize / 3,trackSize, trackSize / 3);
		}
	}

	private static void drawTracklayer(int x, int y, Graphics g) {
		g.setColor(Color.BLUE);
		g.fillRect(x,y,trackSize,trackSize);
		g.fillRect(x - trackSize, y + trackSize / 2, trackSize, trackSize / 2);
		g.setColor(Color.DARK_GRAY);
		for (int i = 0; i < tracksInStore; ++i) {
			g.fillRect(x - trackSize, 
					   y + (trackSize / 2) - (trackSize / 8) - (i * trackSize) / 6, 
					   trackSize * 5 / 6, 
					   trackSize / 8);
		}
	}
	
	public static void drawScreen(Graphics g) {
		if (!inited) return;
		Rectangle bound = g.getClipBounds();
		int shift;
		int startTrack;
		int endTrack;
		if (position * trackSize < bound.width / 2) {
			shift = 0;
			startTrack = 0;
			endTrack = bound.width / trackSize + 1;
		} else {
			shift = -((position * trackSize) - (bound.width / 2));
			startTrack = -((bound.width / 2) / trackSize) + position - 1;
			endTrack = ((bound.width / 2) / trackSize) + position + 1;
		}
		startTrack = Math.max(startTrack, 0);
		endTrack =   Math.min(endTrack, railroad.length - 1);
		drawTracklayer(shift + position * trackSize, bound.height - 2*trackSize, g);
		for (int i = startTrack; i <= endTrack; ++i) {
			drawTrack(railroad[i], shift + i * trackSize, bound.height - trackSize,g);
			if (i % 4 == 0) {
				g.setColor(Color.BLACK);
				g.drawString(new Integer(i).toString(), shift + i * trackSize + trackSize / 3, bound.height - trackSize / 2);
			}
		}
		boolean allCorrect = true;
		for (int i = 0; i < railroad.length; ++i) {
			if (railroad[i] == BAD_TRACK || railroad[i] == NO_TRACK) {
				allCorrect = false;
				break;
			}
		}
		if (allCorrect && position <= 1) {
			g.drawString("Complete", trackSize * 3, trackSize);
		}
	}
}
