package ru.ifmo.tracklayer.runner;

import com.evelopers.unimod.runtime.*;
import com.evelopers.unimod.runtime.logger.*;
import com.evelopers.unimod.runtime.context.*;
import org.apache.commons.logging.*;

import ru.ifmo.tracklayer.co.Screen;
import ru.ifmo.tracklayer.co.Tracklayer;
import ru.ifmo.tracklayer.ep.ScreenEventProvider;
import ru.ifmo.tracklayer.ep.TracklayerEventProvider;

import com.evelopers.common.exception.*;

public class TracklayerRunner {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		TracklayerRunner runner = new TracklayerRunner();
		runner.run();
	}

	private class ExitListener extends AbstractEventProcessorListener {
		private boolean done;

		public void stateMachineCameToFinalState(StateMachineContext context, StateMachinePath path,
				StateMachineConfig config) {
			if (path.isRoot()) done = true;
		}
	}

	private void run() {
		ModelEngine me;
		try {
			ControlledObjectsManager b = new ControlledObjectsManager() {
				Tracklayer o1 = new Tracklayer();
				Screen o2 = new Screen();

				public void init(ModelEngine engine) {
				}

				public void dispose() {
				}

				public ControlledObject getControlledObject(String coName) {
					if (coName.equals("o1")) return o1;
					if (coName.equals("o2")) return o2;
					return null;
				}
			};
			me = ModelEngine.createStandAlone(new QueuedHandler(),
					new ru.ifmo.tracklayer.runner.Model1EventProcessor(), b,
					new EventProvidersManager() {
				ScreenEventProvider p1 = new ScreenEventProvider();
				TracklayerEventProvider p2 = new TracklayerEventProvider();
				
				public void init(ModelEngine engine) throws CommonException {
					p1.init(engine);
					p2.init(engine);
				}

				public void dispose() {
					p1.dispose();
					p2.dispose();
				}

				public EventProvider getEventProvider(String epName) {
					if (epName.equals("ep1")) return p1;
					if (epName.equals("ep2")) return p2;
					return null;
				}
			});
			me.getEventProcessor().addEventProcessorListener(new SimpleLogger(LogFactory.getLog(SimpleLogger.class)));
			ExitListener l1 = new ExitListener();
			me.getEventProcessor().addEventProcessorListener(l1);
			me.start();

			// wait for final state
			while (!l1.done) {
				Thread.sleep(10);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
